#if !defined( BITUTILS_H_INCLUDED ) //{
#  define BITUTILS_H_INCLUDED
/*@Description
FILE DESCRIPTION:

    This file defines a series of inlined functions and utilities used for
    manipulating data types at the byte and bit level, e.g. setting and
    accessing bits in a byte, or getting low and high order bytes and word
    from larger integer types.

    If templates are supported, template versions of the bit functions are
    provided, so all integer types may be used.
*/

#include "isamdefs.h"

    /**********************/
    /* Byte Manipulations */
    /**********************/

typedef unsigned char       BYTE;       // these three typedefs from windows.h
typedef unsigned int        WORD;
typedef unsigned long       DWORD;

inline DWORD
MakeLong( WORD lo, WORD hi )
{
    return (long) (((WORD)(lo)) | (((DWORD)((WORD)(hi))) << 16));
}

inline WORD
MakeWord( BYTE lo, BYTE hi )
{
    return (WORD) (lo | (((WORD) hi) << 8 ));
}

inline WORD
LoWord( long l )
{
    return (WORD) l;
}

inline WORD
HiWord( long l )
{
    return (WORD) (((DWORD)(l) >> 16) & 0xFFFF);
}

inline BYTE
LoByte( WORD w )
{
    return (BYTE) w;
}

inline BYTE
HiByte( WORD w )
{
    return  (BYTE)(((WORD)(w) >> 8) & 0xFF);
}


    /*********************/
    /* Bit Manipulations */
    /*********************/

#if defined( __BCPLUSPLUS__ )       //{
#   if defined( __TEMPLATES__ )     //{

template < class N >
inline void
ClearBit( N & c, int bit_pos )
{
    if ( bit_pos < 0 || bit_pos > ( sizeof(N) * 8 ))
        return;

    c &= ~( 1 << bit_pos );
}

template < class N >
inline boolean
GetBit( N c, int bit_pos )
{
    if ( bit_pos < 0 || bit_pos > ( sizeof(N) * 8 ))
        return FALSE;

    return c & ( 1 << bit_pos );
}

template < class N >
inline void
SetBit( N & c, int bit_pos )
{
    if ( bit_pos < 0 || bit_pos > ( sizeof(N) * 8 ))
        return;

    c |= ( 1 << bit_pos );
}

template < class N >
inline void
SetBit( N & c, int bit_pos, boolean state )
{
    if ( state )
        SetBit( c, bit_pos );
    else
        ClearBit( c, bit_pos );
}

#   endif                           //}
#else                               //}{

inline void
ClearBit( unsigned char & c, int bit_pos )
{
    if ( bit_pos < 0 || bit_pos > 7 )
        return;

    c &= ~( 1 << bit_pos );
}

inline boolean
GetBit( unsigned char c, int bit_pos )
{
    if ( bit_pos < 0 || bit_pos > 7 )
        return FALSE;

    return c & ( 1 << bit_pos );
}

inline void
SetBit( unsigned char & c, int bit_pos )
{
    if ( bit_pos < 0 || bit_pos > 7 )
        return;

    c |= ( 1 << bit_pos );
}

inline void
SetBit( unsigned char & c, int bit_pos, boolean state )
{
    if ( state )
        SetBit( c, bit_pos );
    else
        ClearBit( c, bit_pos );
}

#endif                              //}

#endif //} #if !defined( BITUTILS_H_INCLUDED )

