#include "Application.h"

// The const flags values
CApp::flags CApp::fl_null     =  0;      // optional 
CApp::flags CApp::fl_req      =  2;      // required     (otherwise optional)
CApp::flags CApp::fl_arg      =  4;      // argument     (otherwise option)
CApp::flags CApp::fl_array    = 16;      //
CApp::flags CApp::fl_used     = 32;
CApp::flags CApp::fl_multiple = 64;

int     CApp::m_argc;
char**  CApp::m_argv;
CApp::CArg**  CApp::m_args; 
int     CApp::m_args_index; 
int     CApp::m_args_max_index;
int     CApp::m_flags;

typedef CApp::CArg* CApp_CArg_ptr;
//--------------------------------------------------------------------
CApp::CArg::CArg(
	   char* a_fn,  
	   char* a_m, 
	   char* a_ev,
	   flags a_f 
    ) 
    : m_name(a_fn), m_message(a_m), m_flags(a_f), m_env_variable(a_ev)
{
    CApp::add(this);
}

//..................................................................

void
CApp::add(CArg* arg)
{
    assert(m_args_index <= m_args_max_index);
    if (m_args_index == m_args_max_index) {
	// We run put of space, copy them
	if (m_args_index == 0) m_args_index = 1;
	CApp_CArg_ptr* tmp = new CApp_CArg_ptr[2 * m_args_index];
	for (int i = 0; i < m_args_index; i++) {
	    tmp[i] = m_args[i];
	}
	delete m_args;
	m_args = tmp;
	m_args_max_index = 2 * m_args_max_index;
    }

    // We now have enough space
    assert(m_args_index < m_args_max_index);
    m_args[m_args_index] = arg;
    m_args_index++;  
}


//..................................................................
void 
CApp::CArg::print_usage(ostream& str)
{
    if (!(this->m_flags & CApp::fl_req)) str << "[";
    if (!(this->m_flags & CApp::fl_arg)) str << "-";
    str  << this->m_name  << type();
    if (this->m_flags & CApp::fl_array) str << " ...";
    if (!(this->m_flags & CApp::fl_req)) str << "]";
    str << " ";
}

//....................................................................

void 
CApp::CArg::print_usage_extended(ostream& str)
{
    str << "\t\"";
    if (!(m_flags & fl_arg)) str << "-";
    str << this->m_name << "\"" << " ";
    str << this->m_message  << " ";
    if (m_env_variable != NULL) {
	str << " ENV: $" <<	this->m_env_variable;
    }
    if (!( m_flags & CApp::fl_req)) {
	str << " Default: ";
	str << this;
    }
    str << endl;
}

//**********************************************************************

void 
CApp::parse_argument()
{
    for (int i = 0; i < m_args_index; i++) {
	if (m_args[i]->is_option()) continue;
	if (m_args[i]->is_used()) continue;  
	
	m_args[i]->set_value_from_lexeme(m_argv[0]); 
	m_args[i]->m_flags |= fl_used;
	return;
    }

    strstream str;
    str << "Unexepected argument " << m_argv[0] << '\0';
    if (!(m_flags & IgnoreUnknownArgs))
	print_usage_and_exit(str.str());
}

//.....................................................

void
CApp::parse_internal()
{
   while (m_argc-- > 0) {
       if (*m_argv[0] == '-') {
	   parse_flags();
       } else {
	   parse_argument();
       }
       m_argv++;
   }

   for (int i = 0; i < m_args_index; i++) {
       if (!(m_args[i]->is_used()) && m_args[i]->is_required()) {
	   strstream str;
	   str << "missing required argument. name: " << m_args[i]->m_name << '\0';
	   print_usage_and_exit(str.str());
       }
   }
}

//......................................................

void 
CApp::parse_flags()
{	
    for (int i = 0; i < m_args_index; i++) {
	if (m_args[i]->is_argument()) continue;
	if (m_args[i]->is_used()) continue;
	// Get the lexeme of this argument if any
	char* lexeme = NULL;
	if ((strlen(m_args[i]->name()) == 1) &&
	    ((m_args[i]->name())[0] == m_argv[0][1])) {
	    if (m_argv[0][2] == '\0') {
		if (!m_args[i]->has_length()) {
		    lexeme = "";
		} else {
		    m_argv++; m_argc--;
		    lexeme = m_argv[0];
		}
	    } else {
		lexeme = &m_argv[0][2];
	    }	
	} else if (strcmp(m_args[i]->name(), 
			  &m_argv[0][1]) == 0) {
	    if (!m_args[i]->has_length()) {
		lexeme = "";
	    } else {
		m_argv++; m_argc--;
		lexeme = m_argv[0];
	    }
	} else {
	    continue;
	}
	
	// Now parse the lexeme and store it at the appropriate
	assert(lexeme != NULL);
	m_args[i]->set_value_from_lexeme(lexeme);
	m_args[i]->m_flags |= CApp::fl_used;
	return;
		    
    }
    if (i >= m_args_index) {
	strstream str;
	str << "Unknown option " << m_argv[0] << ends;
	if (!(m_flags & IgnoreUnknownOptions))
	    print_usage_and_exit(str.str());
    }
}

//...............................................

void 
CApp::set_env_values()
{
    for (int i = 0; i < m_args_index; i++) {
	if (m_args[i]->m_env_variable != NULL) {
	    char* lexeme = getenv(m_args[i]->m_env_variable);
	    if (lexeme != NULL) {
		m_args[i]->set_value_from_lexeme(lexeme); 
	    }	
	}
    }

}

//..............................................

void 
CApp::print_usage_and_exit(const char* reason, int a_code)
{
    strstream str;
    // Print the first line
    str << "Error: " << reason << endl;
    str << "Usage: ";
    for (int i = 0; i < m_args_index; i++) {
	m_args[i]->print_usage(str);
    }
    str << endl;
    
    // Print the explanations
    for (i = 0; i < m_args_index; i++) {
	m_args[i]->print_usage_extended( str);
    }
    str << '\0';
    fatal(str.str(), a_code);
}

//.................................................

CApp::CApp(int a_flags)
{
    m_args_index = 0;
    m_args_max_index = 1;
    m_args = new CApp_CArg_ptr[1];
    m_flags = a_flags;
}

//...................................................

ostream& 
operator<<(ostream& str, CApp::CArg* a)
{
    a->print(str);
    return str;
}

