////////////////////////////////////////////////////////
//                  top of file
////////////////////////////////////////////////////////

#include "afx.h"
#include "afxdb.h"
#include "stdafx.h"

#include <string.h>
#include <memory.h>

#include "dynacols.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#else
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

////////////////////////////////////////////////////////
// simple macros for checking SQLfuncx return values
//
#define SQL_SUCCESSFUL(rc) (!((rc)>>1))
#define SQL_NOTSUCCESSFUL(rc) ((rc)>>1)

////////////////////////////////////////////////////////
// CColumns implementation
//
IMPLEMENT_DYNAMIC(CColumns, CRecordset)

////////////////////////////////////////////////////////
// constructor
//
CColumns::CColumns(CDatabase* pDatabase): CRecordset(pDatabase)
{
    //{{AFX_FIELD_INIT(CColumns)
    //}}AFX_FIELD_INIT
    m_nFields = 0;
    m_bWasPrepared = FALSE;

    // buffer for building insert strings from column data
    m_iBufferSize = 4096;
    m_iBufferInc  = 1024;
    m_iBufferSafety = m_iBufferSize - m_iBufferInc;
    m_pcInsertBuffer = new char[m_iBufferSize];
}


////////////////////////////////////////////////////////
// destructor: free all heap memory allocated by this
//
CColumns::~CColumns()
{
   UINT     kounter;
   CColumnInfo* pColInfo;
   
   for (kounter = 0; kounter < m_nFields; kounter++)
   {
      pColInfo =  (CColumnInfo*) m_ColumnList.GetAt(kounter);
      delete pColInfo->pData;                      
      delete pColInfo;
   }
   m_ColumnList.RemoveAll();
   delete m_pcInsertBuffer;
}


////////////////////////////////////////////////////////
// Function illustrates one way to work with this class
//
LPCSTR  CColumns::GetInsertString( char* pcName )
{
    CByteArray* poByteArray;
    CString*    poString;
    CColumnInfo*    pxInfo;
    CTime*      poTime;
    char        achTemp[255];
    int         i;
    int         iSize = 0;
    int         iLoop = 0;
    int         iStmt = 0;
    int         iMargin = 0;
    int         iLen = 0;
    BOOL        fFirstTime = TRUE;
    char*       pcTemp;

    iSize = m_ColumnList.GetSize();

    iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "(" );
    iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "'%s', ", pcName );    

    for( i=0; i<iSize; i++ )
    {       
       pxInfo = (CColumnInfo*)m_ColumnList[i];
       if( fFirstTime )
          fFirstTime = FALSE;
       else
          iStmt += wsprintf( &m_pcInsertBuffer[iStmt], ", " );
            
       switch( pxInfo->fSqlType )
       {
          case SQL_NUMERIC:
          case SQL_DECIMAL:
          case SQL_FLOAT:
          case SQL_REAL:
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "%f", 
                                 (double)*(float*)pxInfo->pData );
          break;
          case SQL_INTEGER:
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "%li", 
                                 *(long*)pxInfo->pData );
          break;            
          case SQL_SMALLINT:
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "%i", 
                                  *(int*)pxInfo->pData );
          break;    
          case SQL_BIGINT:
          case SQL_DOUBLE:
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "%f", 
                                 *(double*)pxInfo->pData );
          break;            
          case SQL_TINYINT:
          case SQL_BIT:
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "%u", 
                                 (int)*(UCHAR*)pxInfo->pData );
          break;            
          case SQL_CHAR:  
          case SQL_VARCHAR:
          case SQL_LONGVARCHAR:
               poString = (CString*)pxInfo->pData;                                                
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "'" );
               strcpy(achTemp, 
                     (char*)(const char*)poString->GetBuffer(poString->GetLength()) );
               poString->ReleaseBuffer( -1 ); 
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], achTemp );
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "'" );
          break;
          case SQL_DATE:
          case SQL_TIME:
          case SQL_TIMESTAMP:
               poTime = (CTime*)pxInfo->pData;
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "'" );
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], 
                                 (const char*)poTime->Format( "%m/%d/%y %H:%M:%S" ) );
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "'" );
          break;
          case SQL_BINARY:
          case SQL_VARBINARY:
               poByteArray = (CByteArray*)pxInfo->pData;
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "0x" );
               for (iLoop = 0; iLoop = poByteArray->GetSize(); iLoop++)
                    iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "%i",
                                      (int)(poByteArray->GetAt(iLoop)) ); 
               iStmt += wsprintf( &m_pcInsertBuffer[iStmt], "'" );
          break;
          case SQL_LONGVARBINARY:
               // make sure there is enough room in the buffer
               iLen = LongBinary( (CLongBinary*)pxInfo->pData, NULL );
               iStmt += iLen;
               if( iStmt > m_iBufferSafety )
               {
                   // extend the string buffer
                   iMargin = m_iBufferSize - m_iBufferSafety;
                   m_iBufferSize = iStmt + 2 * iMargin;
                   m_iBufferSafety = iStmt + iMargin;
                   pcTemp = new char[m_iBufferSize];
                   strcpy( pcTemp, m_pcInsertBuffer );
                   delete m_pcInsertBuffer;
                   m_pcInsertBuffer = pcTemp;
                }

                LongBinary( (CLongBinary*)pxInfo->pData, 
                            &m_pcInsertBuffer[iStmt - iLen] );
          break;                    
          default:
              VERIFY(FALSE);
          break;
       }   // end switch            
       // after adding each field, examine the working buffer
       // to see if the buffer has to be re-alloced larger
       if( iStmt >= m_iBufferSafety )
       {
           if( iStmt > m_iBufferSize )
               // a heap buffer overwrite has occurred
               return NULL;
           // extend the string buffer
           iMargin = m_iBufferSize - m_iBufferSafety;
           m_iBufferSize = iStmt + 2 * iMargin;
           m_iBufferSafety = iStmt + iMargin;
           pcTemp = new char[m_iBufferSize];
           strcpy( pcTemp, m_pcInsertBuffer );
           delete m_pcInsertBuffer;
           m_pcInsertBuffer = pcTemp;
       }
    }        
    iStmt += wsprintf( &m_pcInsertBuffer[iStmt], ")" );
    return m_pcInsertBuffer;
}


////////////////////////////////////////////////////////
//   Handles special work related to SQLLongBinary type
//
int CColumns::LongBinary( CLongBinary* poLb, char* pcBuf )
{
    int     iLen;
    char*   pcData;
    char*   pcEnd;

    if( poLb->m_dwDataLength == 0L )
        return 0;

    iLen = 2 + 2 * poLb->m_dwDataLength;
    if( pcBuf == NULL )
        return iLen;

    // Determine if field has an ODD number of bytes
    // AND more than 255 bytes. If so, can not include 
    // this field in the insert string; because the 
    // insert at the far-end WILL fail. Fallback: 
    // Put in a string representing an "empty field"
    // log the error and move on. 
    //
    // see Transact-SQL Language manual under topic:
    //     Text/Image Datatypes for more info
    //  
    if ((poLb->m_dwDataLength%2 == 1)      
      &&(poLb->m_dwDataLength >= 255)) 
    {
          strcpy(pcBuf, "0x00");
          LogError (THIS_FILE, __LINE__, 
                    "DATA ERROR: Bad data was read from 'image' field");
          return iLen;
    }
    
    if ((poLb->m_dwDataLength < 255)     
      &&(poLb->m_dwDataLength%2 == 1) )  
    {
        // if less than 255 bytes AND odd number of bytes,
        // need to pad field with leading zero
        strcpy (pcBuf, "0x0" );       
        pcBuf += 4;                   
    }
    else
    {
        strcpy( pcBuf, "0x" );       
        pcBuf += 3;                   
    }    

    pcData = (char*)::GlobalLock( poLb->m_hData );
    pcEnd = pcData + poLb->m_dwDataLength;
    while( pcData < pcEnd )
        pcBuf += wsprintf( pcBuf, "%2X", *pcData++ );

    ::GlobalUnlock( poLb->m_hData );    
    pcBuf += wsprintf( pcBuf, "'" );    
    return iLen;
}


////////////////////////////////////////////////////////
// IMPORTANT NOTICE: This class will THROW a db exception 
//              IF:
//          1) This was constructed with a NULL CDatabase
//           -- or --
//          2) This was constructed with a CLOSED CDatabase
//
// Since this class is fully dynamic and does not know what
// it must bind to until run-time, we have to build a list
// of CColumnInfo classes (column descriptor classes) that is 
// used to steer logic in this class. That work is done here. 
//
BOOL CColumns::PrepareToOpen(UINT nOpenType,        /* = snapshot     */
                            LPCSTR lpszSQL,         /* = NULL         */ 
                            DWORD dwOptions)        /* = none         */
{
   BOOL bReturn;
   
   // with this class, we want to force the 
   // passage of a select statement in Open
   ASSERT(lpszSQL != NULL);

   // Store Query into member variable.
   m_Query = lpszSQL;
                                              //    see IMPORTANT NOTICE above
   bReturn = MakeColumnListFromQuery();       //  <<---< can throw DB exception
   
   m_bWasPrepared = bReturn;
   return bReturn;
}

////////////////////////////////////////////////////////
// used to open this recordset
// OUTPUT: TRUE if base class Open was successful 
//         FALSE otherwise
//
BOOL CColumns::OpenDynamicRowset(CString* cStrQuery)
{
   BOOL bReturn;
   
   // check to see if this object was properly prepared
   //
   if ( !m_bWasPrepared )
        ThrowDBException(SQL_INVALID_HANDLE); //  <<--< it wasn't, so toss cookies...

   // force synchronous mode for ODBC
   m_pDatabase->SetSynchronousMode(TRUE);
    
   bReturn = CRecordset::Open(CRecordset::snapshot, 
                              (LPCSTR) m_Query->GetBuffer(m_Query->GetLength()), 
                              CRecordset::none);
   m_Query->ReleaseBuffer();
   if (bReturn)
   {  // force result-rows into local cache
      MoveLast();
      MoveFirst();
   }
   return bReturn;
}


////////////////////////////////////////////////////////
// FORCE USE of OpenDynamicRowset
//
BOOL CColumns::Open(UINT nOpenType, LPCSTR lpszSQL, DWORD dwOptions)
{
#ifdef DEBUG
   return FALSE;
#else
   ThrowDBException(SQL_INVALID_HANDLE);
#endif
}


////////////////////////////////////////////////////////
// returns "ERROR", as this function has no use in this class
//
CString CColumns::GetDefaultConnect()
{
    return "ERROR";
}


////////////////////////////////////////////////////////
// returns the select statement used in this class
//
CString CColumns::GetDefaultSQL()
{
    return m_Query;
}


////////////////////////////////////////////////////////
// uses the ColumnInfoList to steer the dynamic binding
// that will be used in this
//
void CColumns::DoFieldExchange(CFieldExchange* pFX)
{
#define MAX_READ  32767

   UINT     kounter = 0;
   CColumnInfo* pColInfo;

   //{{AFX_FIELD_MAP(CColumns)
   //}}AFX_FIELD_MAP
   pFX->SetFieldType(CFieldExchange::outputColumn);

   while (kounter < m_nFields)
   {
      pColInfo = (CColumnInfo*) m_ColumnList.GetAt(kounter);
      switch (pColInfo->fSqlType)
      {
          // character family
          case SQL_CHAR:
          case SQL_VARCHAR:
                 RFX_Text(pFX, (const char *)pColInfo->szColumnName, 
                               *((CString *) pColInfo->pData ) );
          break;            
          case SQL_LONGVARCHAR:
                 RFX_Text(pFX, (const char *)pColInfo->szColumnName, 
                               *((CString *) pColInfo->pData), MAX_READ );
          break;
            
          // Integer family
          case SQL_BIT:
                 RFX_Bool(pFX, (const char *)pColInfo->szColumnName, 
                               *((BOOL *) pColInfo->pData) );
          break;            
          case SQL_TINYINT:
                 RFX_Byte(pFX, (const char *)pColInfo->szColumnName, 
                               *((unsigned char *) pColInfo->pData) );
          break;            
          case SQL_SMALLINT:
                 RFX_Int(pFX, (const char *)pColInfo->szColumnName, 
                              *((int *) pColInfo->pData) );
          break;
          case SQL_INTEGER:
                 RFX_Long(pFX, (const char *)pColInfo->szColumnName, 
                               *((long *) pColInfo->pData) );
          break;
          case SQL_BIGINT:
                 RFX_Double(pFX, (const char *)pColInfo->szColumnName, 
                                *((double *) pColInfo->pData) );
          break;

          // Floating-point type family and Fixed-point exact numerics.  
          //
          case SQL_NUMERIC:
          case SQL_DECIMAL:
          case SQL_REAL:
          case SQL_FLOAT:
                 RFX_Single(pFX, (const char *)pColInfo->szColumnName, 
                                 *((float *) pColInfo->pData) );
          break;            
          case SQL_DOUBLE:
                 RFX_Double(pFX, (const char *)pColInfo->szColumnName, 
                                 *((double *) pColInfo->pData) );
          break;
                            
          // Binary types
          //
          case SQL_BINARY:
          case SQL_VARBINARY:
                 RFX_Binary(pFX, (const char *)pColInfo->szColumnName, 
                                 *((CByteArray *) pColInfo->pData) );
          break;            
          case SQL_LONGVARBINARY:
                 RFX_LongBinary(pFX, (const char *)pColInfo->szColumnName, 
                                     *((CLongBinary *) pColInfo->pData) );
          break;
                    
          // Date/Time family
          //
          case SQL_DATE:
          case SQL_TIME:
          case SQL_TIMESTAMP:
                 RFX_Date(pFX, (const char *)pColInfo->szColumnName, 
                               *((CTime *) pColInfo->pData) );
          break;
            
          // We're in trouble if we arrive here
          //
          default:
             ASSERT(0);
             break;
      }    //end switch
   kounter ++;
   }      // end of while loop  
}


////////////////////////////////////////////////////////
//
// finds the WHERE clause used in the query handed to this
// and replaces the WHERE clause with "WHERE 1 = 0" in 
// order to request an EMPTY cursor from the server. This 
// is because at the stage where this function is called,
// this class is merely interested in discovering the 
// meta-data returned from the query, not the data itself;
// and using this query is a way of requesting no rows from
// the server, while still getting access to the meta-data
//
////////////////////////////////////////////////////////
BOOL  CColumns::ReplaceWhereClause()
{
    USHORT    usNotPossible = 10;
    USHORT    usCopyBytes = 0;
    int       iBufferTop = 0;
    LPSTR     lpstrTarget = "WHERE 1 = 0";
    LPSTR     lpstrSubject;
    char*     pTemp;    
    
    // copy and upcase the query to a copy that can have its where clause messed up
    m_QueryNoWhere = m_Query;

    lpstrSubject=  m_QueryNoWhere.GetBuffer(m_QueryNoWhere.GetLength());
  
    // Check for bogus string...
    if (strlen ( (const char *) lpstrSubject) < usNotPossible)
    {
        m_QueryNoWhere.ReleaseBuffer();
        return FALSE;
    }
    
    // Locate "where"
    if ( NULL == strstr (lpstrSubject, (const char *) "WHERE")
      && NULL == strstr (lpstrSubject, (const char *) "where") )
          iBufferTop = strlen ( (const char *) lpstrSubject);
    else
    {
       pTemp = strstr (lpstrSubject, (const char *) "where");
       iBufferTop = pTemp - lpstrSubject;
    }      

    usCopyBytes = strlen( (const char *) lpstrTarget);
    memcpy (lpstrSubject + iBufferTop, lpstrTarget, usCopyBytes + 1);

    m_QueryNoWhere.GetBufferSetLength(strlen(lpstrSubject));    
    return TRUE;
}


////////////////////////////////////////////////////////
// adds a CColumnInfo class to the CPtrArray: m_ColumnList
//
void  CColumns::AddColumnInfoToList(CColumnInfo* pColInfo)
{
   m_ColumnList.SetAt(m_nFields, pColInfo);
   m_nFields++;
}


////////////////////////////////////////////////////////
// This function allocates (and initializes) the proper 
// type of storage location for each field returned 
// from the discovery query.
//
BOOL  CColumns::AllocStorageForColumnVar(CColumnInfo* pColInfo)
{
   switch (pColInfo->fSqlType)
   {
       // character family
       case SQL_CHAR:
       case SQL_VARCHAR:
       case SQL_LONGVARCHAR:
             pColInfo->pData = (void *) new CString;
             *((CString *) pColInfo->pData ) = "";
       break;
            
       // Integer family
       case SQL_BIT:
             pColInfo->pData = (void *) new BOOL;
             *((BOOL *) pColInfo->pData ) = FALSE;
       break;            
       case SQL_TINYINT:
             pColInfo->pData = (void *) new unsigned char;
             *((unsigned char *) pColInfo->pData ) = '\0';
       break;            
       case SQL_SMALLINT:
             pColInfo->pData = (void *) new int;
             *((int *) pColInfo->pData ) = 0;
       break;
       case SQL_INTEGER:
             pColInfo->pData = (void *) new long;
             *((long *) pColInfo->pData ) = 0;
       break;
       case SQL_BIGINT:
             pColInfo->pData = (void *) new double;
             *((double *) pColInfo->pData ) = 0;
       break;

       // Floating-point type family and Fixed-point exact numerics.
       case SQL_NUMERIC:
       case SQL_DECIMAL:
       case SQL_REAL:
       case SQL_FLOAT:
             pColInfo->pData = (void *) new float;
             *((float *) pColInfo->pData ) = (float) 0;
       break;            
       case SQL_DOUBLE:
             pColInfo->pData = (void *) new double;
             *((double *) pColInfo->pData ) = 0.0;
       break;
                            
       // Binary types
       case SQL_BINARY:
       case SQL_VARBINARY:
             pColInfo->pData = (void *) new CByteArray;
             ((CByteArray *) pColInfo->pData )->SetSize(1,1);
       break;            
       case SQL_LONGVARBINARY:
             pColInfo->pData = (void *) new CLongBinary;
             ((CLongBinary *) pColInfo->pData )->m_hData = NULL;
             ((CLongBinary *) pColInfo->pData )->m_dwDataLength = 0;
       break;
                    
      // Date/Time family
      case SQL_DATE:
      case SQL_TIME:
      case SQL_TIMESTAMP:
             pColInfo->pData = (void *) new CTime;
             *((CTime *) pColInfo->pData ) = 0;
      break;
            
      // We're in trouble if we arrive here
      //
      default:
         ASSERT(0);
         return FALSE;
      break;
   }    //end switch
   return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
//
// Map a SQL type to the C equivalent.
//
BOOL  CColumns::MapColumnSQLTypeToCType(CColumnInfo* pColInfo)
{
   switch (pColInfo->fSqlType)
   {
        case SQL_CHAR:
        case SQL_VARCHAR:
        case SQL_LONGVARCHAR:
        case SQL_DECIMAL:
        case SQL_NUMERIC:
        case SQL_BIGINT:
            pColInfo->fCType = SQL_C_CHAR;
        break;
            
        case SQL_BIT:
            pColInfo->fCType = SQL_C_BIT;
        break;
            
        case SQL_TINYINT:
            pColInfo->fCType = SQL_C_TINYINT;
        break;
            
        case SQL_SMALLINT:
            pColInfo->fCType = SQL_C_SHORT;
        break;
            
        case SQL_INTEGER:
            pColInfo->fCType = SQL_C_LONG;
        break;
            
        case SQL_REAL:
            pColInfo->fCType = SQL_C_FLOAT;
        break;
            
        case SQL_FLOAT:
        case SQL_DOUBLE:
            pColInfo->fCType = SQL_C_DOUBLE;
        break;
            
        case SQL_BINARY:
        case SQL_VARBINARY:
        case SQL_LONGVARBINARY:
            pColInfo->fCType = SQL_C_BINARY;
        break;
            
        case SQL_DATE:
            pColInfo->fCType = SQL_C_DATE;
        break;
            
        case SQL_TIME:
            pColInfo->fCType = SQL_C_TIME;
        break;
            
        case SQL_TIMESTAMP:
            pColInfo->fCType = SQL_C_TIMESTAMP;
        break;
            
        // We're in trouble if we arrive here
        //
        default:
           ASSERT(0);
           pColInfo->fCType = SQL_C_DEFAULT;
           return FALSE;
        break;
   }    //end switch
   return TRUE;
}


////////////////////////////////////////////////////////
// Decides based on the sql type what parameters would
// be required to use the type for table create string.
//
BOOL CColumns::GetCreateParamCount (CColumnInfo* pColInfo)
{
    switch(pColInfo->fSqlType) 
    {
        // These types are fixed size
        //
        case SQL_BIT:
        case SQL_TINYINT:
        case SQL_SMALLINT:
        case SQL_INTEGER:
        case SQL_BIGINT:
        case SQL_REAL:
        case SQL_FLOAT:
        case SQL_DOUBLE:
        case SQL_DATE:
        case SQL_TIME:
        case SQL_TIMESTAMP:
            pColInfo->iParamCount = 0;
        break;

        // These types are (almost always) variable width
        //
        case SQL_CHAR:
        case SQL_VARCHAR:
        case SQL_LONGVARCHAR:
        case SQL_BINARY:
        case SQL_VARBINARY:
        case SQL_LONGVARBINARY:
            pColInfo->iParamCount = 1;
        break;

        // These types can be sized and scaled
        //
        case SQL_NUMERIC:
        case SQL_DECIMAL:
            pColInfo->iParamCount = 2;
        break;

        // We're in trouble if we arrive here
        //
        default:
           ASSERT(0);
           pColInfo->iParamCount = 0;
           return FALSE;
    }
    return TRUE;
}


////////////////////////////////////////////////////////
// uses the CColumnInfo's SQL_TYPE and precision info fields
// to create a ddl-string that can be used to copy this
// table schema 
//
BOOL CColumns::GenerateTableCreationString()
{
   char     chBuffer[1024];
   UINT     kounter = 1;
   CColumnInfo* pColInfo;
   
   m_TableSchema = "( ";
   
   while (kounter < m_nFields)
   {
       pColInfo = (CColumnInfo*) m_ColumnList.GetAt(kounter);

       switch( pColInfo->fSqlType )
       {
           case SQL_CHAR:  
              m_TableSchema  += pColInfo->szColumnName;
              wsprintf( chBuffer, " char(%ld) ", pColInfo->cbPrecision );
              m_TableSchema += chBuffer;
           break;
        
           case SQL_FLOAT:
           case SQL_DOUBLE:
           case SQL_NUMERIC:
           case SQL_REAL:
           case SQL_DECIMAL:
              m_TableSchema += pColInfo->szColumnName;
              m_TableSchema += " float ";
           break;
                             
           case SQL_INTEGER:
           case SQL_BIGINT:
              m_TableSchema += pColInfo->szColumnName;
              m_TableSchema += " int ";
           break;
        
           case SQL_SMALLINT:
              m_TableSchema += pColInfo->szColumnName;
              m_TableSchema += " smallint ";
           break;
        
           case SQL_VARCHAR:
              m_TableSchema += pColInfo->szColumnName;
              wsprintf( chBuffer, " varchar(%ld) ", pColInfo->cbPrecision );
              m_TableSchema += chBuffer;
           break;
        
           case SQL_DATE:
           case SQL_TIME:
           case SQL_TIMESTAMP:
              m_TableSchema += pColInfo->szColumnName;
              m_TableSchema += " datetime ";
           break;                                  

           case SQL_LONGVARCHAR:
              m_TableSchema += pColInfo->szColumnName;
              m_TableSchema += " text ";
           break;
        
           case SQL_BINARY:
              m_TableSchema += pColInfo->szColumnName;
              wsprintf( chBuffer, " binary(%ld) ", pColInfo->cbPrecision );
              m_TableSchema += chBuffer;
           break;
        
           case SQL_VARBINARY:
              m_TableSchema += pColInfo->szColumnName;
              wsprintf( chBuffer, " varbinary(%ld) ", pColInfo->cbPrecision );
              m_TableSchema += chBuffer;
           break;
        
           case SQL_LONGVARBINARY:
              m_TableSchema += pColInfo->szColumnName;
              m_TableSchema += " image ";
           break;
        
           case SQL_BIT:
              m_TableSchema += pColInfo->szColumnName;
              m_TableSchema += " bit ";
           break;

           case SQL_TINYINT:
              m_TableSchema += pColInfo->szColumnName;
              m_TableSchema += " tinyint ";
           break;
        
           // we're in trouble if we land in here
           default:
              ASSERT(0);
              return FALSE;
           break;
       }
       if ( (BOOL)pColInfo->fNullable)
             m_TableSchema += " NULL ";
       else
             m_TableSchema += " NOT NULL ";
       kounter++;
       if (m_nFields != kounter)
          m_TableSchema += " , ";
   }
   m_TableSchema += " )";
   return TRUE;
}       
    

////////////////////////////////////////////////////////
// issues the schema-only query to the database, then 
// loops through the column results, getting data from 
// each column in order to build up the CColumnInfo descriptor.
//
BOOL CColumns::MakeColumnListFromQuery()
{
   BOOL       bRtn=FALSE; // Assume we'll fail; allow reselect
   RETCODE    rc;         // Return code from ODBC functions
   CColumnInfo*   pColInfo;   // Data type output for each column
   int        index;      // Index for columns
   short      iCols;      // the number of columns in this query
   HSTMT      hStmt;      // allocated and used in this function


   // check to see if this object was constructed sans open CDatabase
   if ( (m_hstmt == SQL_NULL_HSTMT) || (!m_pDatabase->IsOpen()) )
        ThrowDBException(SQL_INVALID_HANDLE); 
        //  <<--< it was, so toss cookies...
    
   // safe to continue, so create a meta-data only query
   ReplaceWhereClause();
    
   // Get a HSTMT for use in this function
   rc = ::SQLAllocStmt( m_pDatabase->m_hdbc, &hStmt );
   if (SQL_NOTSUCCESSFUL(rc))
   {
      DecodeSQLError(rc, m_pDatabase->m_hdbc, hStmt, THIS_FILE, __LINE__);
      goto cleanup;
   }

   // Execute the query and retrieve the results
   rc = ::SQLExecDirect(hStmt, 
                       (unsigned char*)
                        m_QueryNoWhere.GetBuffer(m_QueryNoWhere.GetLength()), 
                        m_QueryNoWhere.GetLength());
   if(SQL_NOTSUCCESSFUL(rc)) 
   {
       DecodeSQLError(rc, m_pDatabase->m_hdbc, hStmt, THIS_FILE, __LINE__);
       m_QueryNoWhere.ReleaseBuffer();
       goto cleanup;
    }
    m_QueryNoWhere.ReleaseBuffer();
    
   // how many columns are there in this select?
   rc = ::SQLNumResultCols( hStmt, &iCols);
   if(SQL_NOTSUCCESSFUL(rc)) 
   {
       DecodeSQLError(rc, m_pDatabase->m_hdbc, hStmt, THIS_FILE, __LINE__);
       goto cleanup;
   }
    
    // make the CPtrArray large enough to hold all of the CColumnInfo *'s
    m_ColumnList.SetSize(iCols);

   // now loop through the column set gathering facts about each one
   // to store in a CColumnInfo class for each column. All the CColumnInfo 
   // classes will be added to the m_ColumnList 
   for(index = 0;  index < iCols;  index++) 
   {
       pColInfo = new CColumnInfo;
       if (pColInfo == NULL)
           goto cleanup;      
       pColInfo->nColInRslt = index +1;

       // get the SQL_xxx type (and related info) for the column
       rc = ::SQLDescribeCol(hStmt,
                            (UWORD)(index + 1), 
                             NULL, 
                             0, 
                             NULL,
                             &pColInfo->fSqlType,
                             &pColInfo->cbPrecision,
                             &pColInfo->ibScale,
                             &pColInfo->fNullable);
       if(SQL_NOTSUCCESSFUL(rc))
       {
          DecodeSQLError(rc, m_pDatabase->m_hdbc, hStmt, THIS_FILE, __LINE__);
          goto cleanup;
       }
                
       // Get the named type of the column for mapping
       //
       rc = ::SQLColAttributes(hStmt,
                              (UWORD)(index + 1),
                              SQL_COLUMN_TYPE_NAME,           
                              &pColInfo->szNamedType,
                              sizeof(pColInfo->szNamedType),
                              NULL,
                              NULL);
       if(SQL_NOTSUCCESSFUL(rc))
       {
          DecodeSQLError(rc, m_pDatabase->m_hdbc, hStmt, THIS_FILE, __LINE__);
          goto cleanup;
       }
                
       // Get the display size in case this column gets
       // converted to char
       //
       rc = ::SQLColAttributes(hStmt,
                              (UWORD)(index + 1),
                              SQL_COLUMN_DISPLAY_SIZE,
                              NULL,
                              sizeof(pColInfo->cbDspSize),
                              NULL,
                              &pColInfo->cbDspSize);
       if(SQL_NOTSUCCESSFUL(rc))
       {
          DecodeSQLError(rc, m_pDatabase->m_hdbc, hStmt, THIS_FILE, __LINE__);
          goto cleanup;
       }

       // Get the name of this column
       //
       rc = ::SQLColAttributes(hStmt,
                              (UWORD)(index + 1),
                              SQL_COLUMN_NAME,
                              &pColInfo->szColumnName,
                              sizeof(pColInfo->szColumnName),
                              NULL,
                              NULL);
       if(SQL_NOTSUCCESSFUL(rc))
       {
          DecodeSQLError(rc, m_pDatabase->m_hdbc, hStmt, THIS_FILE, __LINE__);
          goto cleanup;
       }
                
       // Get the name of the table containing this column
       //
       rc = ::SQLColAttributes(hStmt,
                              (UWORD)(index + 1),
                              SQL_COLUMN_TABLE_NAME,
                              &pColInfo->szFromTable,
                              sizeof(pColInfo->szFromTable),
                              NULL,
                              NULL);

       if(SQL_NOTSUCCESSFUL(rc))
       {
          DecodeSQLError(rc, m_pDatabase->m_hdbc, hStmt, THIS_FILE, __LINE__);
          goto cleanup;
       }
                
       // now that we have all the info we need on this column
       // finish up by allocating storage for this column's local
       // member variable and adding the finished CColumnInfo to the
       // the column list
       if ( ! AllocStorageForColumnVar(pColInfo) )
              goto cleanup;
       
       if ( !(MapColumnSQLTypeToCType(pColInfo)) 
         || !(GetCreateParamCount(pColInfo))     )
              goto cleanup;

       AddColumnInfoToList(pColInfo);                
   }
   // if we get here, all went well
   if ( GenerateTableCreationString() )
        bRtn = TRUE;
   // Clean up the statement handle
   //
cleanup:
    SQLFreeStmt(hStmt, SQL_CLOSE);
    SQLFreeStmt(hStmt, SQL_UNBIND);    
    return bRtn;
}


////////////////////////////////////////////////////////
// called whenever an SQLfuncx call returns anything 
// other than success in order to discover more infor-
// mation about what went wrong
//
void CColumns::DecodeSQLError( RETCODE rc, HDBC hdbc, 
                               HSTMT hstmt, LPSTR lpstrFile, 
                               UINT uiLine)
{              
    char    chBuf[3*SQL_MAX_MESSAGE_LENGTH];
    UCHAR   uchSqlState[SQL_MAX_MESSAGE_LENGTH];
    UCHAR   uchErrorMsg[SQL_MAX_MESSAGE_LENGTH];
    SDWORD  sNativeError;
    SWORD   sLength;
    int     n;
    
    VERIFY( lpstrFile != NULL );
    
    switch( rc )
    {
        case SQL_NEED_DATA:
           LogError( lpstrFile, uiLine, "SQL returned: SQL_NEED_DATA" );
        break;
        
        case SQL_NO_DATA_FOUND:
           LogError( lpstrFile, uiLine, "SQL returned: SQL_NO_DATA_FOUND");
        break;
        
        case SQL_STILL_EXECUTING:
           LogError( lpstrFile, uiLine, "SQL returned: SQL_STILL_EXECUTING" );
        break;
        
        case SQL_INVALID_HANDLE:
           LogError( lpstrFile, uiLine, "SQL returned: SQL_INVALID_HANDLE" );
        break;
        
        default:
        case SQL_ERROR:
           n = wsprintf( (char*)chBuf, "%s", "SQL ERROR returned: \n" );
           ::SQLError( SQL_NULL_HENV, hdbc, hstmt, uchSqlState,
                   &sNativeError, uchErrorMsg, SQL_MAX_MESSAGE_LENGTH, 
                              &sLength );
           n += wsprintf( (char*)&chBuf[n], 
                                 "SQL State: %s, ", (char*)uchSqlState );      
           n += wsprintf( (char*)&chBuf[n], "msg: %s", (char*)uchErrorMsg );
           LogError( lpstrFile, uiLine, chBuf );
        break;
    }   
}

////////////////////////////////////////////////////////
//                  end of file
////////////////////////////////////////////////////////
