////////////////////////////////////////////////////////
//                  top of file
////////////////////////////////////////////////////////

#ifndef __DYNACOLS_H__
#define __DYNACOLS_H__

#ifndef __AFXWIN_H__
    #error include 'stdafx.h' before including this file for PCH
#endif

#include "afx.h"
#include "stdafx.h"
#include "afxdb.h"

// Column descriptor class
class CColumnInfo 
{
#define MAX_TYPE 35
public:
   // Following is information from the source column
   //
   SWORD   fSqlType;                // SQL_xxxx type of column
   SWORD   fCType;                  // Mapped C type
   UDWORD  bPrecision;              // Column precision
   SWORD   ibScale;                 // Scale of the column
   char    szNamedType[MAX_TYPE];   // named SQL type of the column
   char    szFromTable[MAX_TYPE];   // Name from original table
   SDWORD  cbDspSize;               // Display size if converted to character
   UWORD   nColInRslt;              // Which col in the results list
   char    szColumnName[MAX_TYPE];  // Name of this column
   void *  pData;                   // Memory for the transfer
   SWORD   fNullable;               // Can column be null?
   int     iParamCount;             // # of creation parameters for this column
};

    
class CColumns : public CRecordset
{
DECLARE_DYNAMIC(CColumns)
public:
   CColumns( CDatabase* pDatabase = NULL);
   BOOL PrepareToOpen( UINT nOpenType = snapshot,
                       LPCSTR lpszSQL = NULL, 
                       DWORD dwOptions = none,
                       char* szServerName = NULL);
   BOOL OpenDynamicRowset(CString* );
   ~CColumns();
   LPCSTR  GetInsertString( char* );

// Field/Param Data
//{{AFX_FIELD(CColumns, CRecordset)
//}}AFX_FIELD

// members
public:
    CPtrArray m_ColumnList;
private:
    CString   m_Query;
    CString   m_QueryNoWhere;
    CString   m_TableSchema;
    LPSTR     m_pcInsertBuffer;
    int       m_iBufferSize;
    int       m_iBufferInc; 
    int       m_iBufferSafety;
    BOOL      m_bWasPrepared;

// Implementation
private:
    int   LongBinary( CLongBinary*, char* );
    BOOL  ReplaceWhereClause();
    void  AddColumnInfoToList(CColumnInfo* pColInfo);
    BOOL PrepareToOpen(UINT nOpenType,                  /* = snapshot     */
                       LPCSTR lpszSQL,                  /* = NULL         */ 
                       DWORD dwOptions)                 /* = none         */
    BOOL  AllocStorageForColumnVar(CColumnInfo* pColInfo);
    BOOL  MapColumnSQLTypeToCType(CColumnInfo* pColInfo);    
    BOOL  GetCreateParamCount (CColumnInfo* pColInfo);       
    BOOL  GenerateTableCreationString(void);             
    BOOL  MakeColumnListFromQuery();
    void  DecodeSQLError( RETCODE retCode, HDBC hdbc, HSTMT hstmt, 
                          LPSTR pszFile, UINT uiLine);
protected:
    virtual CString GetDefaultConnect();                // default connection string
    virtual CString GetDefaultSQL();                    // default SQL for Recordset
    virtual void DoFieldExchange(CFieldExchange* pFX);  // RFX support
    virtual BOOL Open(UINT nOpenType,                   /* = snapshot */
                      LPCSTR lpszSQL,                   /* = NULL     */ 
                      DWORD dwOptions);                 /* = none     */
};

#endif

////////////////////////////////////////////////////////
//                  end of file
////////////////////////////////////////////////////////
