#include "objfile.h"
#include "isamdefs.h"

template < class ObjType >
ObjIndexKey ObjFile<ObjType>::sPrimaryKey_;

    /*************/
    /* Selectors */
    /*************/

template < class ObjType >
boolean
ObjFile<ObjType>::IsOpen()
{
    return 0 <= fileId_;
}

template < class ObjType >
int
ObjFile<ObjType>::GetNumIndexes()
{
    return ISAMFile::GetNumKeys( fileId_ );
}

template < class ObjType >
ISAMFile::CountType
ObjFile<ObjType>::GetNumObjects()
{
    return ISAMFile::GetNumRecs( fileId_ );
}

template < class ObjType >
ISAMFile::CountType
ObjFile<ObjType>::GetCurObjNum()
{
    return GetCurObjNum( fileId_ );
}



template < class ObjType >
ISAMError
ObjFile<ObjType>::GetIndexKey( ObjIndexKey & index_buf, int index_num )
{
    return GetIndexKey( fileId_, index_buf, index_num );

}


template < class ObjType >
ISAMError
ObjFile<ObjType>::Build( ISAMFile::LockModes lock_mode,
                         ISAMFile::AccessModes access_mode )
{
    ISAMError ret = OK;
    struct ISAMFile::IndexKey * isam_key;

    if ( fileId_ >= 0 )
    {
        ret = FILE_IS_OPEN;
    }
    else
    {
        if ( 0 == sPrimaryKey_.nfields )
        {
            isam_key = 0;
        }
        else
        {
            isam_key = new struct ISAMFile::IndexKey;

            if ( ConvertKey( sPrimaryKey_, *isam_key ))
                ret = IDX_ILLEGAL_KEY_DESC;
            else
                curIndex_ = sPrimaryKey_;
        }

        if ( OK == ret )
            fileId_ = ISAMFile::Build( isam_key, recSize_, &ret,
                                      lock_mode, access_mode );
        delete isam_key;
    }

    return ret;

}   /* ObjFile<ObjType>::Build( ISAMFile::LockModes, ISAMFile::AccessModes ) */

template < class ObjType >
ISAMError
ObjFile<ObjType>::LockFile()
{
    return ISAMFile::LockFile( fileId_ );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::Open( ISAMFile::LockModes lock_mode,
                        ISAMFile::AccessModes access_mode )
{
    ISAMError ret = OK;

    if ( !IsFileIDValid( fileId_ ))
        fileId_ = Open( &ret, lock_mode, access_mode );

    return ret;
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::Close()
{
    ISAMError ret = ISAMFile::Close( fileId_ );
    InvalidateID( &fileId_ );
    return ret;
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::Flush()
{
    return ISAMFile::Flush( fileId_ );
}


    // index manipulation
    // ==================

template < class ObjType >
ISAMError
ObjFile<ObjType>::AddIndex( ObjIndexKey & index_key )
{
    return AddIndex( fileId_, index_key );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::DelIndex( ObjIndexKey & index_key )
{
    return DelIndex( fileId_, index_key );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::SelectIndex( ObjIndexKey & index_key,
                               ObjType * search_obj,
                               ISAMFile::SearchModes search_mode )
{
    return SelectIndex( fileId_, index_key, search_obj, search_mode );
}


        // Read/Write Access Methods
        // =========================

template < class ObjType >

ISAMError
ObjFile<ObjType>::GetCur( ObjType & obj, boolean lock )
{
    return GetCur( fileId_, obj, lock );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::GetFirst( ObjType & obj, boolean lock )
{
    return GetFirst( fileId_, obj, lock );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::GetLast( ObjType & obj, boolean lock )
{
    return GetLast( fileId_, obj, lock );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::GetNext( ObjType & obj, boolean lock )
{
    return GetNext( fileId_, obj, lock );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::GetPrev( ObjType & obj, boolean lock )
{
    return GetPrev( fileId_, obj, lock );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::GetEq( ObjType & obj, boolean lock )
{
    return GetEq( fileId_, obj, lock );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::GetEqGt( ObjType & obj, boolean lock )
{
    return GetEqGt( fileId_, obj, lock );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::GetGt( ObjType & obj, boolean lock )
{
    return GetGt( fileId_, obj, lock );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::Goto( ObjType & obj )
{
    return Goto( fileId_, obj );
}


template < class ObjType >
ISAMError
ObjFile<ObjType>::WriteCur( ObjType & obj )
{
    return WriteCur( fileId_, obj );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::Write( ObjType & obj )
{
    return Write( fileId_, obj );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::ReWriteCur( ObjType & obj )
{
    return ReWriteCur( fileId_, obj );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::ReWrite( ObjType & obj, ISAMFile::CountType obj_num )
{
    return ReWrite( fileId_, obj, obj_num );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::ReWrite( ObjType & obj )
{
    return ReWrite( fileId_, obj );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::DeleteCur()
{
    return ISAMFile::DeleteCur( fileId_ );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::DeleteObj( ObjType & obj )
{
    return DeleteObj( fileId_, obj );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::DeleteObj( ISAMFile::CountType obj_num )
{
    return DeleteObj( fileId_, obj_num );
}


template < class ObjType >
ISAMError
ObjFile<ObjType>::UnLockFile()
{
    return ISAMFile::UnLockFile( fileId_ );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::UnLockAllObjects()
{
    return UnLockAllRecs( fileId_ );
}


template < class ObjType >
ObjFile<ObjType>::ObjFile( const char * file_name, ObjFieldInfo * field_info,
                  int num_fields )
:
    ISAMFile( file_name )
{
    fieldInfo_ = field_info;
    numFields_ = num_fields;

    fileId_ = -1;
    recSize_ = fieldInfo_[ numFields_ - 1 ].start +
              fieldInfo_[ numFields_ - 1 ].len;

    isamRec_ = new char[ recSize_ ];

}

template < class ObjType >
ObjFile<ObjType>::~ObjFile()
{
    delete isamRec_;
}

template < class ObjType >
int
ObjFile<ObjType>::SetPrimaryKey( ObjIndexKey & primary_key )
{
    sPrimaryKey_ = primary_key;
    return 0;
}

template < class ObjType >
ObjIndexKey &
ObjFile<ObjType>::GetPrimaryKey()
{
    return sPrimaryKey_;
}

template < class ObjType >
int
ObjFile<ObjType>::ConvertKey( ObjIndexKey & obj_key,
                              ISAMFile::IndexKey & isam_key )
{
    int numRecField,
        ret = 0;

    isam_key.flags = obj_key.flags;
    isam_key.nparts = obj_key.nfields;

    for ( int i = 0;
          i < Min( obj_key.nfields,
                   (short) ISAMFile::MAX_KEY_PARTS );
          i++ )
    {
        numRecField = obj_key.fields[i].fieldNum ;
        if ( numRecField < 0 || numRecField >= numFields_ )
        {
            ret = -1;       // numRecField out of range
            break;
        }

        isam_key.parts[i].start = (short) fieldInfo_[ numRecField ].start;
        isam_key.parts[i].len = (short) fieldInfo_[ numRecField ].len;
        isam_key.parts[i].flags = obj_key.fields[i].flags;
        isam_key.parts[i].type = fieldInfo_[ numRecField ].type;
    }

    return ret;

}       /* ObjFile<ObjType>::ConvertKey( ObjIndexKey &, etc. ) */


template < class ObjType >
int
ObjFile<ObjType>::ConvertKey( ISAMFile::IndexKey & isam_key,
                              ObjIndexKey & obj_key )
{
    int numRecField, ret = 0;

    obj_key.flags = isam_key.flags;
    obj_key.nfields = isam_key.nparts ;

    for ( int i = 0;
          i < Min( obj_key.nfields,
                   (short) ISAMFile::MAX_KEY_PARTS );
          i++ )
    {
            /* Find the right field number for the current isam key part,
            by comparing start location of the field in the isam record
            with the corresponding value in the fieldInfo_ struct.
            */
        for ( numRecField = 0; numRecField < numFields_; numRecField++ )
            if ( fieldInfo_[numRecField].start == isam_key.parts[i].start )
                break;

        if ( numRecField >= numFields_ )
        {
            ret = -1;       // did not find a field corresponding to the
            break;          // isam key part
        }

        obj_key.fields[i].fieldNum = numRecField;
        obj_key.fields[i].flags = isam_key.parts[i].flags;
    }

    return ret;

}       /* ObjFile<ObjType>::ConvertKey( ISAMFile::IndexKey &, etc. ) */

template < class ObjType >
boolean
ObjFile<ObjType>::IsNull( const ObjType & ) const
{
 return TRUE; // no-op default implementation
}

template < class ObjType >
void
ObjFile<ObjType>::MakeNull( ObjType & )
{
 // no-op default implementation
}

    /**********************************/
    /* Private Implementation Methods */
    /**********************************/

    // Selectors
    // =========

template < class ObjType >
ISAMFile::CountType
ObjFile<ObjType>::GetCurObjNum( ISAMFile::AccessID file_id )
{
    return GetCurRecNum( file_id );
}

    // file manipulation
    // =================

template < class ObjType >
ISAMFile::AccessID
ObjFile<ObjType>::Open( ISAMError * err,
                        ISAMFile::LockModes lock_mode,
                        ISAMFile::AccessModes access_mode )
{
    AccessID ret;
    ret = ISAMFile::Open( err, lock_mode, access_mode );
    if ( *err == OK )
        curIndex_ = sPrimaryKey_;

    return ret;

}   /* ObjFile<ObjType>::Open( ISAMFile::LockModes, ISAMFile::AccessModes ) */

template < class ObjType >
ISAMError
ObjFile<ObjType>::Close( ISAMFile::AccessID file_id )
{
    return ISAMFile::Close( file_id );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::Flush( ISAMFile::AccessID file_id )
{
    return ISAMFile::Flush( file_id );
}


    // index manipulation
    // ==================

template < class ObjType >
ISAMError
ObjFile<ObjType>::GetIndexKey( ISAMFile::AccessID file_id,
                               ObjIndexKey & index_buf, int index_num )
{
    ISAMError ret = OK;
    struct ISAMFile::IndexKey isam_index;

    if ( index_num < 0 )
        index_buf = curIndex_;
    else if ( OK == ( ret = ISAMFile::GetIndexKey( file_id, isam_index, index_num )))
        if ( 0 != ConvertKey( isam_index, index_buf ))
            ret = IDX_BAD_KEY;

    return ret;

}       /* ObjFile<ObjType>::GetIndexKey( ObjIndexKey &, int ) */




template < class ObjType >
ISAMError
ObjFile<ObjType>::AddIndex( ISAMFile::AccessID file_id,
                            ObjIndexKey & index_key )
{
    struct ISAMFile::IndexKey isam_idx_key;

    if ( ConvertKey( index_key, isam_idx_key ))
        return IDX_ILLEGAL_KEY_DESC;
    else
        return ISAMFile::AddIndex( file_id, isam_idx_key );

}       /* ObjFile<ObjType>::AddIndex( ObjIndexKey & ) */


template < class ObjType >
ISAMError
ObjFile<ObjType>::DelIndex( ISAMFile::AccessID file_id, ObjIndexKey & index_key  )
{
    struct ISAMFile::IndexKey isam_idx_key;

    if ( ConvertKey( index_key, isam_idx_key ))
        return IDX_ILLEGAL_KEY_DESC;
    else
        return ISAMFile::DelIndex( file_id, isam_idx_key );

}       /* ObjFile<ObjType>::DelIndex( ObjIndexKey & ) */



template < class ObjType >
ISAMError
ObjFile<ObjType>::SelectIndex( ISAMFile::AccessID file_id,
                               ObjIndexKey & index_key,
                               ObjType * search_obj,
                               ISAMFile::SearchModes search_mode )
{
    struct ISAMFile::IndexKey isam_idx_key;
    ISAMError  ret = OK;

    if ( ConvertKey( index_key, isam_idx_key ))
    {
        ret = IDX_ILLEGAL_KEY_DESC;
    }
    else
    {
        curIndex_ = index_key;

        if ( 0 != search_obj )
        {
            if ( ConvObjToRec( *search_obj, isamRec_, &index_key ))
                ret = REC_BAD;
            else ret = ISAMFile::SelectIndex( file_id, isam_idx_key, search_mode,
                                    isamRec_, 0 );
        }
        else
        {
            ret = ISAMFile::SelectIndex( file_id, isam_idx_key, search_mode, 0, 0 );
        }
    }

    return ret;

}       /* ObjFile<ObjType>::SelectIndex( ObjIndexKey &, etc. ) */



    // read/write
    // ==========

template < class ObjType >
ISAMError
ObjFile<ObjType>::GetCur( ISAMFile::AccessID file_id, ObjType & obj,
                          boolean lock )
{
    ISAMError ret;

    ret = ReadCur( file_id, isamRec_, lock );

    if ( ret == OK )
        if ( ConvRecToObj( isamRec_, obj, 0 ))
            ret = REC_BAD;

    return ret;

}       /* ObjFile<ObjType>::GetCur( ISAMFile::AccessID, ObjType &, boolean ) */


template < class ObjType >
ISAMError
ObjFile<ObjType>::GetNext( ISAMFile::AccessID file_id, ObjType & obj, boolean lock )
{
    ISAMError ret;

    ret = ReadNext( file_id, isamRec_, lock );

    if ( ret == OK )
        if ( ConvRecToObj( isamRec_, obj, 0 ))
            ret = REC_BAD;

    return ret;

}       /* ObjFile<ObjType>::GetNext( ISAMFile::AccessID, ObjType &, boolean ) */


template < class ObjType >
ISAMError
ObjFile<ObjType>::GetPrev( ISAMFile::AccessID file_id, ObjType & obj, boolean lock )
{
    ISAMError ret;

    ret = ReadPrev( file_id, isamRec_, lock );

    if ( ret == OK )
        if ( ConvRecToObj( isamRec_, obj, 0 ))
            ret = REC_BAD;

    return ret;

}       /* ObjFile<ObjType>::GetPrev( ISAMFile::AccessID, ObjType &, boolean ) */


template < class ObjType >
ISAMError
ObjFile<ObjType>::GetFirst( ISAMFile::AccessID file_id, ObjType & obj, boolean lock )
{
    ISAMError ret;

    ret = ReadFirst( file_id, isamRec_, lock );

    if ( ret == OK )
        if ( ConvRecToObj( isamRec_, obj, 0 ))
            ret = REC_BAD;

    return ret;

}       /* ObjFile<ObjType>::GetFirst( ISAMFile::AccessID, ObjType &, boolean ) */


template < class ObjType >
ISAMError
ObjFile<ObjType>::GetLast( ISAMFile::AccessID file_id, ObjType & obj, boolean lock )
{
    ISAMError ret;

    ret = ReadLast( file_id, isamRec_, lock );

    if ( ret == OK )
        if ( ConvRecToObj( isamRec_, obj, 0 ))
            ret = REC_BAD;

    return ret;

}


template < class ObjType >
ISAMError
ObjFile<ObjType>::GetEq( ISAMFile::AccessID file_id, ObjType & obj, boolean lock )
{
    ISAMError ret;

    if ( ConvObjToRec( obj, isamRec_, &curIndex_ ))
        ret = REC_BAD;
    else if ( OK == ( ret = ReadEq( file_id, isamRec_, lock )))
        if ( ConvRecToObj( isamRec_, obj, 0 ))
            ret = REC_BAD;

    return ret;
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::GetEqGt( ISAMFile::AccessID file_id, ObjType & obj, boolean lock )
{
    ISAMError ret;

    if ( ConvObjToRec( obj, isamRec_, &curIndex_ ))
        ret = REC_BAD;
    else if ( OK == ( ret = ReadEqGt( file_id, isamRec_, lock )))
        if ( ConvRecToObj( isamRec_, obj, 0 ))
            ret = REC_BAD;

    return ret;
}


template < class ObjType >
ISAMError
ObjFile<ObjType>::GetGt( ISAMFile::AccessID file_id, ObjType & obj, boolean lock )
{
    ISAMError ret;

    if ( ConvObjToRec( obj, isamRec_, &curIndex_ ))
        ret = REC_BAD;
    else if ( OK == ( ret = ReadGt( file_id, isamRec_, lock )))
        if ( ConvRecToObj( isamRec_, obj, 0 ))
            ret = REC_BAD;

    return ret;
}


template < class ObjType >
ISAMError
ObjFile<ObjType>::Goto( ISAMFile::AccessID file_id, ObjType & obj )
{
    return GetEq( file_id, obj, FALSE );
}



template < class ObjType >
ISAMError
ObjFile<ObjType>::WriteCur( ISAMFile::AccessID file_id, ObjType & obj )
{
    ISAMError ret = OK;

    if ( ConvObjToRec( obj, isamRec_ ))
        ret = REC_BAD;
    else
        ret = AddCur( file_id, isamRec_ );

    return ret;
}


template < class ObjType >
ISAMError
ObjFile<ObjType>::Write( ISAMFile::AccessID file_id, ObjType & obj )
{
    ISAMError ret = OK;

    if ( ConvObjToRec( obj, isamRec_ ))
        ret = REC_BAD;
    else
        ret = Add( file_id, isamRec_ );

    return ret;
}


template < class ObjType >
ISAMError
ObjFile<ObjType>::ReWriteCur( ISAMFile::AccessID file_id, ObjType & obj )
{
    ISAMError ret = OK;

    if ( ConvObjToRec( obj, isamRec_ ))
        ret = REC_BAD;
    else
        ret = UpdateCur( file_id, isamRec_ );

    return ret;
}


template < class ObjType >
ISAMError
ObjFile<ObjType>::ReWrite( ISAMFile::AccessID file_id, ObjType & obj, ISAMFile::CountType obj_num )
{
    ISAMError ret = OK;

    if ( ConvObjToRec( obj, isamRec_ ))
        ret = REC_BAD;
    else
        ret = Update( file_id, isamRec_, obj_num );

    return ret;
}


template < class ObjType >
ISAMError
ObjFile<ObjType>::ReWrite( ISAMFile::AccessID file_id, ObjType & obj )
{
    ISAMError ret = OK;

    if ( ConvObjToRec( obj, isamRec_ ))
        ret = REC_BAD;
    else
        ret = Update( file_id, isamRec_ );

    return ret;
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::DeleteCur(  ISAMFile::AccessID file_id )
{
    return ISAMFile::DeleteCur( file_id );
}

template < class ObjType >
ISAMError
ObjFile<ObjType>::DeleteObj( ISAMFile::AccessID file_id, ObjType & obj )
{
    ISAMError ret = OK;

    if ( ConvObjToRec( obj, isamRec_ ))
        ret = REC_BAD;
    else
        ret = Delete( file_id, isamRec_ );

    return ret;
}


template < class ObjType >
ISAMError
ObjFile<ObjType>::DeleteObj( ISAMFile::AccessID file_id, ISAMFile::CountType obj_num )
{
    return Delete( file_id, obj_num );
}
