#include "objfiter.h"

    /****************/
    /* Constructors */
    /****************/

template < class ObjType >
ObjFileIter<ObjType>::ObjFileIter( ObjFile<ObjType> & iter_file )
:
    iterFile_( iter_file )
{
    curIndexKey_ = iterFile_.GetPrimaryKey();
    curObjNum_ = (ISAMFile::CountType) 0;
    iterFile_.InvalidateID( &streamId_ );

}       /*  ObjFileIter<ObjType>::ObjFileIter( ObjFile<ObjType> & ) */

template < class ObjType >
ObjFileIter<ObjType>::~ObjFileIter()
{
    if ( ISAMFile::IsFileIDValid( streamId_ ))
        Close();

}       /* ObjFileIter<ObjType>::~ObjFileIter() */


    /*************/
    /* Selectors */
    /*************/

template < class ObjType >
boolean
ObjFileIter<ObjType>::IsOpen()  { return 0 <= streamId_; }

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::GetLastErr() { return lastError_; }

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::GetIndexKey( ObjIndexKey & key_buf, int index_num )
{
    return iterFile_.GetIndexKey( streamId_, key_buf, index_num );
}


    /*************/
    /* Modifiers */
    /*************/

    // file manipulation
    // =================

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::Open( ISAMFile::LockModes lock_mode = ISAMFile::exclusive,
                         ISAMFile::AccessModes access_mode = ISAMFile::readWrite )
{
    ISAMError ret = FILE_IS_OPEN;

    if ( !ISAMFile::IsFileIDValid( streamId_ ))
    {
        streamId_ = iterFile_.Open( &ret, lock_mode, access_mode );
        if ( OK == ret )
            ret = SelectIndex( curIndexKey_ );
    }

    return ret;
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::Close()
{
    ISAMError ret = iterFile_.Close( streamId_ );
    iterFile_.InvalidateID( &streamId_ );
    return ret;
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::Flush()
{
    return iterFile_.Flush( streamId_ );
}


    // index manipulation
    // ==================

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::AddIndex( ObjIndexKey & index_key )
{
    return iterFile_.AddIndex( streamId_, index_key );
}


template < class ObjType >
ISAMError
ObjFileIter<ObjType>::DelIndex( ObjIndexKey & index_key )
{
    return iterFile_.DelIndex( streamId_, index_key );
}


/*********
    If the index is not built already for the file, this method fails.  Use
    AddIndex() to create the index first.
************/

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::SelectIndex(
                            ObjIndexKey & index_key,
                            ObjType * search_obj,
                            ISAMFile::SearchModes search_mode )
{
    ISAMError ret;
    ret = iterFile_.SelectIndex( streamId_, index_key,
                                search_obj, search_mode );

    if ( OK == ret )
    {
        curIndexKey_ = index_key;               // store new index
        curObjNum_ = iterFile_.GetCurObjNum();  // store cur recpos
    }

    return ret;

}       /* ObjFileIter<ObjType>::SelectIndex( etc. ) */


    // Read/Write
    // ==========

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::GetCur( ObjType & obj_buf, boolean lock )
{
    return iterFile_.GetCur( streamId_, obj_buf, lock );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::GetFirst( ObjType & obj_buf, boolean lock )
{
    return iterFile_.GetFirst( streamId_, obj_buf, lock );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::GetLast( ObjType & obj_buf, boolean lock )
{
    return iterFile_.GetLast( streamId_, obj_buf, lock );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::GetNext( ObjType & obj_buf, boolean lock )
{
    return iterFile_.GetNext( streamId_, obj_buf, lock );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::GetPrev( ObjType & obj_buf, boolean lock )
{
    return iterFile_.GetPrev( streamId_, obj_buf, lock );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::GetEq( ObjType & obj_buf, boolean lock )
{
    return iterFile_.GetEq( streamId_, obj_buf, lock );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::GetEqGt( ObjType & obj_buf, boolean lock )
{
    return iterFile_.GetEqGt( streamId_, obj_buf, lock );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::GetGt( ObjType & obj_buf, boolean lock )
{
    return iterFile_.GetGt( streamId_, obj_buf, lock );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::Goto( ObjType & obj )
{
    return iterFile_.Goto( streamId_, obj );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::WriteCur( ObjType & obj )
{
    return iterFile_.WriteCur( streamId_, obj );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::Write( ObjType & obj )
{
    return iterFile_.Write( streamId_, obj );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::ReWriteCur( ObjType & obj )
{
    return iterFile_.ReWriteCur( streamId_, obj );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::ReWrite( ObjType & obj, ISAMFile::CountType obj_num )
{
    return iterFile_.ReWrite( streamId_, obj, obj_num );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::ReWrite( ObjType & obj )
{
    return iterFile_.ReWrite( streamId_, obj );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::DeleteCur()
{
    return iterFile_.DeleteCur( streamId_ );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::DeleteObj( ObjType & obj )
{
    return iterFile_.DeleteObj( streamId_, obj );
}

template < class ObjType >
ISAMError
ObjFileIter<ObjType>::DeleteObj( ISAMFile::CountType obj_num )
{
    return iterFile_.DeleteObj( streamId_, obj_num );
}


    // operators
    // =========

template < class ObjType >
ObjType &
ObjFileIter<ObjType>::operator++()
{
    if ( OK!= ( lastError_ = iterFile_.GetNext( streamId_, retObject_, FALSE )))
    {
     iterFile_.MakeNull( retObject_ );
    }
    return retObject_;
}

template < class ObjType >
ObjType &
ObjFileIter<ObjType>::operator++(int)
{
    ObjType dead;

    if ( OK == ( lastError_ = iterFile_.GetCur( streamId_, retObject_, FALSE )))
        lastError_ = iterFile_.GetNext( streamId_, dead, FALSE );

    if ( OK != lastError_ )
    {
     iterFile_.MakeNull( retObject_ );
    }
    return retObject_;
}

template < class ObjType >
ObjType &
ObjFileIter<ObjType>::operator--()
{
    if ( OK != ( lastError_ = iterFile_.GetPrev( streamId_, retObject_, FALSE )))
    {
     iterFile_.MakeNull( retObject_ );
    }
    return retObject_;
}

template < class ObjType >
ObjType &
ObjFileIter<ObjType>::operator--(int)
{
    ObjType dead;

    if ( OK == ( lastError_ = iterFile_.GetCur( streamId_, retObject_, FALSE )))
        lastError_ = iterFile_.GetPrev( streamId_, dead, FALSE );

    if ( OK != lastError_ )
    {
     iterFile_.MakeNull( retObject_ );
    }
    return retObject_;
}

template < class ObjType >
ObjType &
ObjFileIter<ObjType>::operator ()()
{
    if ( OK != ( lastError_ = iterFile_.GetCur( streamId_, retObject_, FALSE )))
    {
     iterFile_.MakeNull( retObject_ );
    }
    return retObject_;
}


template < class ObjType >
boolean
ObjFileIter<ObjType>::IsNull( const ObjType & obj ) const
{
 return iterFile_.IsNull( obj );
}

/*********
    This assigment operator copies the access stream attributes from the
    other iterator, i.e. the object file reference, the current index key,
    and the current object reference.  It does not copy the stream access
    ID, so that changes in one will not affect the other.

    If this object is currently open, it is closed, and reopened with the
    changed stream attributes.  Otherwise this iterator remains closed, and
    the new stream attributes take effect when it is opened.
************/

template < class ObjType >
ObjFileIter<ObjType> &
ObjFileIter<ObjType>::operator=(
                ObjFileIter<ObjType> & other )
{
    curIndexKey_  = other.curIndexKey_;
    curObjNum_    = other.curObjNum_;
    iterFile_     = other.iterFile_;

    if ( ISAMFile::IsFileIDValid( streamId_ ))
    {
        Close();
        lastError_ = Open();     // reopen so new attributes take effect.
    }

    return *this;

}       /* ObjFileIter<ObjType>::operator=( etc. ) */
