#if !defined( OBJFITER_H_INCLUDED ) //{
#   define OBJFITER_H_INCLUDED

#include "objfile.h"

template < class ObjType >
class ObjFileIter {
public:

/*@Constructors */

    ObjFileIter( ObjFile<ObjType> & iter_file );
    ~ObjFileIter();

/*@Selectors */

    boolean IsOpen();
    ISAMError  GetLastErr();
    ISAMError  GetIndexKey( ObjIndexKey & key_buf, int index_num = -1 );

/*@Modifiers */

    // file manipulation
    // =================

    ISAMError Open( ISAMFile::LockModes lock_mode = ISAMFile::exclusive,
                         ISAMFile::AccessModes access_mode = ISAMFile::readWrite );
    ISAMError Close();
    ISAMError Flush();         // flushes write buffers to disk

    // index manipulation
    // ==================

    ISAMError AddIndex( ObjIndexKey & index_key );
    ISAMError DelIndex( ObjIndexKey & index_key );
    ISAMError SelectIndex( ObjIndexKey & index_key,
                                ObjType * search_obj = NULL,
                                ISAMFile::SearchModes search_mode =
                                                         ISAMFile::equal );
    // read/write
    // ==========

    ISAMError GetCur(   ObjType & obj_buf, boolean lock = FALSE );
    ISAMError GetFirst( ObjType & obj_buf, boolean lock = FALSE );
    ISAMError GetLast(  ObjType & obj_buf, boolean lock = FALSE );
    ISAMError GetNext(  ObjType & obj_buf, boolean lock = FALSE );
    ISAMError GetPrev(  ObjType & obj_buf, boolean lock = FALSE );
    ISAMError GetEq(    ObjType & obj_buf, boolean lock = FALSE );
    ISAMError GetEqGt(  ObjType & obj_buf, boolean lock = FALSE );
    ISAMError GetGt(    ObjType & obj_buf, boolean lock = FALSE );

    ISAMError Goto( ObjType & obj );    // >= according to cur index

    ISAMError WriteCur(   ObjType & obj );
    ISAMError Write(      ObjType & obj );
    ISAMError ReWriteCur( ObjType & obj );
    ISAMError ReWrite(    ObjType & obj, ISAMFile::CountType obj_num );
    ISAMError ReWrite(    ObjType & obj );
    ISAMError DeleteCur();
    ISAMError DeleteObj(  ObjType & obj );
    ISAMError DeleteObj(  ISAMFile::CountType obj_num );

    // operators
    // =========

    ObjType & operator++();             // GetNext
    ObjType & operator++(int);          // postfix - not as efficient
    ObjType & operator--();             // GetPrev
    ObjType & operator--(int);          // postfix - not as efficient
    ObjType & operator ()();            // GetCur;
    ObjFileIter<ObjType> & operator=( ObjFileIter<ObjType> & other );
    boolean IsNull( const ObjType & obj ) const; // to check EOF with operators

private:

    ObjIndexKey         curIndexKey_;
    ISAMFile::CountType curObjNum_;
    ISAMError      lastError_;     // valid only with methods which
                                        // don't return error codes
    ObjFile<ObjType> &  iterFile_;
    ISAMFile::AccessID  streamId_;
    ObjType             retObject_;     // used by operators ++, --, (),
                                        // and casting operators
};

#endif //} #if !defined( OBJFITER_H_INCLUDED ) //{
