#if !defined( PLAYEVNT_H_INCLUDED ) //{
#   define PLAYEVNT_H_INCLUDED

#include "isamdefs.h"

class PlayEvent {
public:
    typedef long SeqNumType;
    typedef int  IDType;

    enum PEStatus {
            NotAvail,
            Avail,
            Cued,
            Playing,
            Paused,
            Cancelled,
            PlayedOK,
            PlayedLate,
            PlayedEarly,
            PlayError
    };

    enum Fields {
            SeqNum = 0,
            ID,
            Status,
            Len,
            Name
    };

    enum Formats {
            DEFAULT     = 0x00,
            NO_MINS     = 0x01, // Len only
            ABREV       = 0x01, // Status
            BINARY      = 0x01, // SeqNum, ID, and SourceID only
            ALL_UPPER   = 0x10, // all fields
            ALL_LOWER   = 0x20,
            PAD_NULLS   = 0x40,
    };

    enum StrErrors {
            OK,
            OUT_OF_RANGE,
            BAD_FORMAT,         // can't parse
            BAD_VALUE,          // doesn't make sense
    };

/* Constructors */

    PlayEvent();
    PlayEvent( IDType new_ID );
    ~PlayEvent();

/* Selectors */

    SeqNumType       seqNum() const;
    IDType           id() const;
    PEStatus            status() const;
    int              length() const;
    const String &      name() const;

    char * GetFieldStr( char * buf, Fields field,
                        int size = 0, Formats format_flags = DEFAULT );

/* Modifiers */

    int seqNum( SeqNumType new_seq_num );
    int id( IDType new_id );
    int status( PEStatus new_status );
    int length( int new_len );
    int name( const String & new_name );

    int PutFieldStr( const char * new_val, Fields field,
                     int size = 0, Formats format_flags = DEFAULT );

private:
    SeqNumType      seqNum_;         // sequence number in playlist
    IDType          id_;             // unique id
    PEStatus        status_;
    int             length_;         // in seconds
    String          name_;           // of spot, commercial, or song
};

#endif //} #if !defined( PLAYEVNT_H_INCLUDED )

